***This do file produces Table 1 in Chetty, Friedman, and Rockoff (2016) "Using Lagged Outcomes to Evaluate Bias in Value-Added Models"
***This version: January 23, 2016
***The code contains three parts: (1) simulate data on test scores; (2) estimate value-added; (3) construct Table 1

clear
set mem 1g
set seed 717

global user user

global root "/Users/$user/Dropbox/IRS_NYC_dropbox/aea_p&p"

***SET UP PARAMETERS FOR SIMULATION***
*  Simulation models repeated years in a single grade at each track.
global min_year       = 1992   // Start year (for readability)
global n_school       = 10000  // Number of tracks, equivalent to 5 tracks at 2000 schools 
global n_year         = 6     // Number of years
global max_year = ${min_year} + ${n_year} - 1
global n_rooms        = 4      // Number of classrooms per track and grade
global n_class        = 25     // Number of students per class
global sd_va          = 0.10   // Standard deviation of value added
global sd_class_shock = 0.08   // Standard deviation of classroom-level shocks
global sd_ability     = 0.88   // SD of level differences across students
global sorting_level  = 0.25
global sd_gain        = 0.15   // SD f trend differences accross students

*  Key parameters for controlling simulation
global sorting		  = 0   //  Controls true bias in VA
global sd_ty_shock    = 0.06   // Standard deviation of track-by-year shocks
global rho            = 1   //  Across grade correlation of TY shock

/*  Base values are sd_ty_shock = 0.04, sorting = 0 or 0.25  */

*  Other helpful globals
global l2res_sd		  = sqrt(0.25 - ${sd_ty_shock}^2 - ${sd_gain}^2) 
global lres_sd		  = sqrt(0.25 - ${sd_ty_shock}^2) 
global res_sd		  = sqrt(0.25 - ${sd_ty_shock}^2 - ${sd_class_shock}^2 - ${sd_va}^2 - ${sd_gain}^2)

*** Set up data
set obs  `= ${n_school} * ${n_rooms} * ${n_year} * ${n_class}'
g track  = ceil(_n / (${n_class} * ${n_rooms} * ${n_year}))
g year   = mod(ceil(_n / (${n_rooms} * ${n_class})) - 1 , ${n_year}) + ${min_year}
g lag_class_number = year - ${min_year} + 1
replace year = ${max_year} - 1 if year < ${max_year}
g class = ceil(_n /${n_class})
g teacher   = mod(class - 1 , ${n_rooms}) + 1 + (track-1)*${n_rooms} 
g student = _n

* Generate true VA and class shocks
g class_shock   = rnormal(0, ${sd_class_shock}) if mod(_n,${n_class}) == 1
replace class_shock = class_shock[_n-1]			if class_shock == .
g va_true       = rnormal(0,${sd_va})           if lag_class_number==1 & mod(_n,${n_class}) == 1
replace va_true = va_true[_n-${n_rooms}*${n_class}] if lag_class_number>1 & mod(_n,${n_class}) == 1
replace va_true = va_true[_n-1]                 if va_true==.

* Generate and propogate track-year shocks
g ty_shock = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == 1
replace ty_shock = ty_shock[_n-1] if ty_shock == . & year < ${max_year}
replace ty_shock = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == ///
	(${n_rooms} * ${n_class} * (${n_year}-1)+1)
replace ty_shock = ty_shock[_n-1] if ty_shock == . & year == ${max_year}
g l_ty_shock = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == 1
replace l_ty_shock = l_ty_shock[_n-1] if l_ty_shock == . & year < ${max_year}
g l_ty_shock_alt = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == ///
	(${n_rooms} * ${n_class} * (${n_year}-1)+1)
replace l_ty_shock_alt = l_ty_shock_alt[_n-1] if l_ty_shock_alt == . & year == ${max_year}
replace l_ty_shock = ${rho}*ty_shock[_n-(${n_rooms}*${n_class})] + sqrt(1-${rho}^2)*l_ty_shock_alt if year == ${max_year}
g l2_ty_shock = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == 1
replace l2_ty_shock = l2_ty_shock[_n-1] if l2_ty_shock == . & year < ${max_year}
g l2_ty_shock_alt = rnormal(0,${sd_ty_shock}) if mod(_n , ${n_rooms} * ${n_class} * ${n_year}) == ///
	(${n_rooms} * ${n_class} * (${n_year}-1)+1)
replace l2_ty_shock_alt = l2_ty_shock_alt[_n-1] if l2_ty_shock_alt == . & year == ${max_year}
replace l2_ty_shock = ${rho}*l_ty_shock[_n-(${n_rooms}*${n_class})] + sqrt(1-${rho}^2)*l2_ty_shock_alt if year == ${max_year}

* Generate abilities and test scores
g ability = rnormal(0,sqrt(${sd_ability}^2-(${sd_va}*${sorting_level})^2)) + va_true*${sorting_level}
g trend = rnormal(0,sqrt(${sd_gain}^2-(${sd_va}*${sorting})^2)) + va_true*${sorting}
g l2_score = ability - trend + l2_ty_shock + rnormal(0,${l2res_sd})
g l_score = ability + l_ty_shock + rnormal(0,${lres_sd})
g score = ability + trend + ty_shock + class_shock + va_true + rnormal(0,${res_sd})

*** Estimate VA
*Make dataset balanced panel
replace score = . if l_score == .
g gain = score - l_score
g gain_oos = gain + l_ty_shock - l_ty_shock_alt   // oos = Out of Sample
g lag_gain = l_score - l2_score
g lag_gain_oos = lag_gain + l2_ty_shock - 2*l_ty_shock + l_ty_shock_alt   // oos = Out of Sample

* Estimate variance components
qui sum gain
global tot_var = r(Var)
preserve
collapse score gain lag_gain l_score l2_score va_true trend ability ty_shock l_ty_shock, by(teacher track year lag_class_number)
g block = track + (year-1900)/100 + lag_class_number/1000
sort block teacher
tsset block teacher
corr gain l.gain if year < ${max_year}, c
global ty_var = r(cov_12)
sort teacher lag_class_number
tsset teacher lag_class_number
corr gain l2.gain if year < ${max_year}, c
global teach_var = (r(cov_12) - ${ty_var})
qui sum gain
global ctot_var = r(Var)
global ind_var   = (${n_class} / (${n_class} - 1)) * (${tot_var} - ${ctot_var})
global class_var = ${tot_var} - ${ind_var} - ${teach_var} - ${ty_var}

* Construct shrinkage factor and VA estimate
g temp = 1 if year < ${max_year}  & ~missing(score)
bys teacher: egen temp1 = sum(temp)
g classes_taught = temp1
g shrinkage = ${teach_var} / (${ty_var} + ${teach_var} + ${class_var} / temp1 + ${ind_var} / (${n_class} * temp1)) 
g temp2 = gain*temp
bys teacher: egen temp3 = mean(temp2) 
g va_est = temp3 * shrinkage if year==${max_year}
g va_raw = temp3
drop temp*
keep teacher va_est va_raw shrinkage year classes_taught
keep if year == ${max_year}
tempfile va_temp
save `va_temp'
restore
merge m:1 teacher year using `va_temp'
saveold $root/analysis/tab1_data.dta, replace


*** Table 1: Effects of Teacher VA on Test Score Gains
cap log close
log using "$root/paper/figs/table1_ftest", replace
	
eststo clear
	foreach v in 4 {
	* Col 1:
		qui eststo: reg va_true va_est, cl(track)
	* Col 2:
		qui eststo: reg lag_gain va_est, cl(track)
		areg lag_gain, a(teacher) // F-test
	* Col 3
		qui eststo: reg lag_gain va_true, cl(track)
	* Cols 4-5:
		qui eststo: reg gain_oos va_est, cl(track)  
		qui eststo: reg gain_oos va_est lag_gain_oos, cl(track)  
	}
	
esttab using "$root/paper/figs/table1.csv", replace se nostar
log close
